#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "tinlib.h"
#include "rgbtools.h"


void set_creation_choices_sensitive(GtkWidget *main_window,gboolean sensitive)
/* *******************************************************************
* ******************************************************************* */
{ 
  GtkWidget *create_tin_button;
  create_tin_button = gtk_object_get_data(GTK_OBJECT(main_window),"create_tin_button");
  gtk_widget_set_sensitive(create_tin_button,sensitive);
}/* set_creation_choices_sensitive */

void set_output_choices_sensitive(GtkWidget *main_window,gboolean sensitive)
/* *******************************************************************
* ******************************************************************* */
{ 
  GtkWidget *save_tin_file,*save_outline_file;

  save_tin_file=gtk_object_get_data(GTK_OBJECT(main_window),"save_tin_file");
  gtk_widget_set_sensitive(save_tin_file,sensitive);
  save_outline_file=gtk_object_get_data(GTK_OBJECT(main_window),"save_outline_file");
  gtk_widget_set_sensitive(save_outline_file,sensitive);
}/* set_output_choices_sensitive */

/* *******************************************************************
*  Begin support for file selection.
******************************************************************* */

void on_filesel_ok_button_clicked(GtkButton *button,gpointer user_data)
/* *******************************************************************
* ******************************************************************* */
{ 
  gtk_object_set_data (GTK_OBJECT(user_data), "answer", "ok"); 
}
void on_filesel_cancel_button_clicked(GtkButton *button,gpointer user_data)
/* *******************************************************************
* ******************************************************************* */
{ 
  gtk_object_set_data (GTK_OBJECT(user_data), "answer", "cancel");
}
bool do_file_selection (GtkWidget *main_window,char *title,char *filename)
/* *******************************************************************
*  This window automatically hangs right in this function
* until the user clicks on an answer.  Allows for writing of simple,
* sequential code.
* ******************************************************************* */
{
  GtkWidget *filew;
  GtkWidget *ok_button,*cancel_button;
  char *fname,*answer;
  bool retval;

  filew = gtk_file_selection_new (title);
  gtk_container_set_border_width (GTK_CONTAINER (filew), 10);

  ok_button = GTK_FILE_SELECTION (filew)->ok_button;
  gtk_widget_show (ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  cancel_button = GTK_FILE_SELECTION (filew)->cancel_button;
  gtk_widget_show (cancel_button);
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_object_set_data (GTK_OBJECT(filew), "answer", "dunno");
  gtk_signal_connect_object(GTK_OBJECT(filew), "delete_event",
                      GTK_SIGNAL_FUNC(on_filesel_cancel_button_clicked),
                      GTK_OBJECT(filew));
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_filesel_ok_button_clicked),
                      GTK_OBJECT(filew));
  gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_filesel_cancel_button_clicked),
                      GTK_OBJECT(filew));

  /* Force parent to be background, and insensitive. */
  gtk_window_set_transient_for(GTK_WINDOW(filew),GTK_WINDOW(main_window));
  gtk_widget_set_sensitive(main_window,FALSE);

  gtk_widget_show(filew);
  /* here's the loop we deliberately hang in. */
  do {
    while (g_main_iteration(FALSE));
    answer=gtk_object_get_data (GTK_OBJECT (filew), "answer");
  }while (strcasecmp(answer,"dunno")==0);

  /* now decipher the answer and return it as boolean and a filename. */
  if (strcasecmp(answer,"ok")==0){
    retval=true;
    fname=gtk_file_selection_get_filename((GtkFileSelection *)filew);
    strcpy(filename,fname);
  }else{
    if (strcasecmp(answer,"cancel")==0){
      retval=false;
    }else{
      printf("file selection window error!\n");
    }
  }
  gtk_widget_destroy(filew); gtk_widget_set_sensitive(main_window,TRUE);
  return(retval);
}/* do_file_selection */

/* *******************************************************************
*  End support for file selection.
******************************************************************* */

void data_free(GtkObject *data_mule)
/* *******************************************************************
*  Free all the xyz and TIN data memory structures.
******************************************************************* */
{
  PListT *points=NULL;
  tin_contextT *tin_context=NULL;

  points=gtk_object_get_data(data_mule, "points");
  if (points!=NULL){ /* Dispose of any prior data. */
    FreePList(points); free(points); points=NULL;
    gtk_object_set_data (data_mule, "points", NULL);
  }
  tin_context=gtk_object_get_data (GTK_OBJECT(data_mule), "tin_context");
  if (tin_context!=NULL){  /* dispose of any old tin structures. */
    tin_destroy(tin_context); 
    free(tin_context); tin_context=NULL;
    gtk_object_set_data (data_mule, "tin_context", NULL);
  }
}/* data_free */

void draw_data(GtkWidget *main_window)
/* *******************************************************************
*  Draw the xyz point data, resized to fit a window.
*  Get the min_max and set the framing values from this window size.  
* Use an rgb array as the buffer.
******************************************************************* */
{
  GtkWidget *tin_drawing_area;
  PListT *points=NULL;
  XYZType *plist,*P;
  int num_points,cnt;
  FrameT fr;
  CubeT *min_max;
  rgbpadT *rgb_pad;
  double drawx,drawy;
  guchar rgb[pixelsize]={0,0,250};

  tin_drawing_area = gtk_object_get_data(GTK_OBJECT(main_window),"tin_drawing_area");
  points = gtk_object_get_data(GTK_OBJECT(main_window), "points");
  if (points!=NULL){
    rgb_pad = gtk_object_get_data(GTK_OBJECT(main_window), "rgb_pad");
    min_max = gtk_object_get_data(GTK_OBJECT(main_window), "min_max");
    memcpy(&fr.Wn,min_max,sizeof(CubeT));
    BorderCube(&fr.Wn,fr.Wn.Sz[0]*0.1);
    SetCube(0,0,0,tin_drawing_area->allocation.width,tin_drawing_area->allocation.height,0,&fr.Vw);
    FlipCubeDim(&fr.Wn,1);/* flip Y coordinate so north is up. */
    OrthoFitND(&fr.Vw,&fr.Wn,2); ResetFrame(&fr);

    rgb_fillpad(rgb_pad,255,255,255);/* blank prior image. */
    num_points=points->NumPoints;
    plist=points->P;
    for (cnt=0;cnt<num_points;cnt++){
      P=&(plist[cnt]); Prj(P->Loc[0],P->Loc[1],&fr,&drawx,&drawy);
      rgb_putpixel(rgb_pad,drawx,drawy,rgb);
    }
    rgb_updatezone(tin_drawing_area, rgb_pad, 0, 0,
                   tin_drawing_area->allocation.width,tin_drawing_area->allocation.height);
  }else{
/*    printf("No TIN created yet.\n"); for debugging. */
  }
}/* draw_data */

void draw_triangles(GtkWidget *main_window)
/* *******************************************************************
*  Draw the TIN triangles after they have been created.
*  Works mostly the same as draw_data and draw_outline.
******************************************************************* */
{
  GtkWidget *tin_drawing_area;
  tin_contextT *tin_context=NULL;
  FrameT fr;
  CubeT *min_max;
  rgbpadT *rgb_pad;
  gsettingsT gs={0,NULL,0,0,0};

  PListT *points=NULL;
  XYZType *plist,*pt1,*pt2,*pt3;
  uint32 tcnt,num_triangles;
  double drawx1,drawy1,drawx2,drawy2,drawx3,drawy3;
  triT *tri;
  flexelT *tri_tree;

  tin_drawing_area = gtk_object_get_data(GTK_OBJECT(main_window),"tin_drawing_area");
  tin_context = gtk_object_get_data(GTK_OBJECT(main_window), "tin_context"); 
  if (tin_context!=NULL){
    points = gtk_object_get_data(GTK_OBJECT(main_window), "points");
    rgb_pad = gtk_object_get_data(GTK_OBJECT(main_window), "rgb_pad");
    min_max = gtk_object_get_data(GTK_OBJECT(main_window), "min_max");
    memcpy(&fr.Wn,min_max,sizeof(CubeT));
    BorderCube(&fr.Wn,fr.Wn.Sz[0]*0.1);
    SetCube(0,0,0,tin_drawing_area->allocation.width,tin_drawing_area->allocation.height,0,&fr.Vw);
    FlipCubeDim(&fr.Wn,1);/* flip Y coordinate so north is up. */
    OrthoFitND(&fr.Vw,&fr.Wn,2); ResetFrame(&fr);

    rgb_setcolor(&gs,0,0,255,1);/* blue triangles */

    plist=points->P;
    tri_tree=&(tin_context->tri_tree);
    num_triangles=tin_context->num_triangles;
    for(tcnt=0;tcnt<num_triangles;tcnt++){
      tri=flexray_lookup(tri_tree,tcnt);
      pt1=&(points->P[tri->vtex[0]]); pt2=&(points->P[tri->vtex[1]]); pt3=&(points->P[tri->vtex[2]]);
      /* project to screen coords. */
      Prj(pt1->Loc[0],pt1->Loc[1],&fr,&drawx1,&drawy1);
      Prj(pt2->Loc[0],pt2->Loc[1],&fr,&drawx2,&drawy2);
      Prj(pt3->Loc[0],pt3->Loc[1],&fr,&drawx3,&drawy3);
      rgb_drawline(rgb_pad,&gs,drawx1,drawy1,drawx2,drawy2);
      rgb_drawline(rgb_pad,&gs,drawx2,drawy2,drawx3,drawy3);
      rgb_drawline(rgb_pad,&gs,drawx3,drawy3,drawx1,drawy1);
    }
    rgb_freecolor(&gs);
    rgb_updatezone(tin_drawing_area, rgb_pad, 0, 0,
                   tin_drawing_area->allocation.width,tin_drawing_area->allocation.height);
  }else{
/*    printf("No TIN created yet.\n"); for debugging. */
  }
}/* draw_triangles */

void draw_outline(GtkWidget *main_window)
/* *******************************************************************
*  Draw the outline of the TIN after it has been created.
*  Works mostly the same as draw_data and draw_triangles.
******************************************************************* */
{
  GtkWidget *tin_drawing_area;
  tin_contextT *tin_context=NULL;
  FrameT fr;
  CubeT *min_max;
  rgbpadT *rgb_pad;
  gsettingsT gs={0,NULL,0,0,0};

  PListT *points=NULL;
  XYZType *plist,*pt1,*pt2;
  int num_points,cnt;
  double drawx1,drawy1,drawx2,drawy2;
  brickptrT loop_wall;
  intdex pdex1,pdex2,brickcnt;
  brickptrT now;
  lineT *this_line;
  
  tin_drawing_area = gtk_object_get_data(GTK_OBJECT(main_window),"tin_drawing_area");
  tin_context = gtk_object_get_data(GTK_OBJECT(main_window), "tin_context"); 
  if (tin_context!=NULL){
    points = gtk_object_get_data(GTK_OBJECT(main_window), "points");
    rgb_pad = gtk_object_get_data(GTK_OBJECT(main_window), "rgb_pad");
    min_max = gtk_object_get_data(GTK_OBJECT(main_window), "min_max");
    memcpy(&fr.Wn,min_max,sizeof(CubeT));
    BorderCube(&fr.Wn,fr.Wn.Sz[0]*0.1);
    SetCube(0,0,0,tin_drawing_area->allocation.width,tin_drawing_area->allocation.height,0,&fr.Vw);
    FlipCubeDim(&fr.Wn,1);/* flip Y coordinate so north is up. */
    OrthoFitND(&fr.Vw,&fr.Wn,2); ResetFrame(&fr);

    rgb_setcolor(&gs,255,0,0,1);

    num_points=points->NumPoints;
    loop_wall=tin_context->loop_wall;
    plist=points->P;
    brickcnt=0; now=loop_wall;
    if (now!=NULL){
      do{
        this_line=now->linptr;
        pdex1=this_line->pdex1; pdex2=this_line->pdex2;
        pt1=&(plist[pdex1]); pt2=&(plist[pdex2]);
        Prj(pt1->Loc[0],pt1->Loc[1],&fr,&drawx1,&drawy1);/* project to screen coords. */
        Prj(pt2->Loc[0],pt2->Loc[1],&fr,&drawx2,&drawy2);
        rgb_drawline(rgb_pad,&gs,drawx1,drawy1,drawx2,drawy2);
        now=now->next; brickcnt++;
      }while(now!=loop_wall);
    }
    rgb_freecolor(&gs);
    rgb_updatezone(tin_drawing_area, rgb_pad, 0, 0,
                   tin_drawing_area->allocation.width,tin_drawing_area->allocation.height);
  }else{
/*    printf("No TIN created yet.\n"); for debugging. */
  }
}/* draw_outline */

void
on_import_xyz_file_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
/* *******************************************************************
******************************************************************* */
{
  bool file_picked;
  char filename[500];
  GtkWidget *main_window;
  PListT *points=NULL;
  CubeT *min_max;
  
  main_window=lookup_widget(GTK_WIDGET(menuitem),"main_window");
  file_picked=do_file_selection(main_window,"Select XYZ File",filename);
  if (file_picked){
    set_output_choices_sensitive(main_window,FALSE);
    data_free(GTK_OBJECT(main_window));
    points=malloc(sizeof(PListT));
    if(tin_read_xyzs(points,filename)){
      gtk_object_set_data(GTK_OBJECT(main_window), "points", points);
      min_max=gtk_object_get_data(GTK_OBJECT(main_window), "min_max");
      SetLimits(points,min_max);/* Get spatial bounds of survey. */
      draw_data(main_window);
      set_creation_choices_sensitive(main_window,TRUE);
    }else{
      printf("Error opening or reading xyz file.\n");
    }
  }
}/* on_import_xyz_file_activate */

void
on_save_tin_file_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
/* *******************************************************************
******************************************************************* */
{
  GtkWidget *main_window;
  tin_contextT *tin_context=NULL;
  char filename[500];
  bool file_picked;
  
  main_window=lookup_widget(GTK_WIDGET(menuitem),"main_window");
  tin_context=gtk_object_get_data (GTK_OBJECT(main_window), "tin_context");
  if (tin_context!=NULL){ /* check if we have a TIN. */
    file_picked=do_file_selection(main_window,"Save TIN File",filename);
    if (file_picked){
      save_tinfile(tin_context,filename);
    }
  }else{
    /* here we print an error message about how you have no data. */
  }
}/* on_save_tin_file_activate */

void
on_save_outline_file_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
/* *******************************************************************
******************************************************************* */
{
  GtkWidget *main_window;
  tin_contextT *tin_context=NULL;
  char filename[500];
  bool file_picked;
  
  main_window=lookup_widget(GTK_WIDGET(menuitem),"main_window");
  tin_context=gtk_object_get_data (GTK_OBJECT(main_window), "tin_context");
  if (tin_context!=NULL){ /* check if we have a TIN. */
    file_picked=do_file_selection(main_window,"Save Outline File",filename);
    if (file_picked){
      tin_save_outline(tin_context,filename);
    }
  }else{
    /* here we print an error message about how you have no data. */
  }
}

void on_create_tin_button_clicked(GtkButton *button,gpointer user_data)
/* *******************************************************************
******************************************************************* */
{
  GtkWidget *main_window,*outline_radius_spinbutton;
  PListT *points=NULL;
  tin_contextT *tin_context=NULL;
  double outrad=1000.0;

  main_window=lookup_widget(GTK_WIDGET(button),"main_window");

  points=gtk_object_get_data(GTK_OBJECT(main_window),"points");
  if (points!=NULL){/* Make sure we have data. */
    outline_radius_spinbutton=gtk_object_get_data(GTK_OBJECT(main_window),"outline_radius_spinbutton");
    outrad = atof(gtk_entry_get_text(GTK_ENTRY(outline_radius_spinbutton)));

    tin_context=gtk_object_get_data (GTK_OBJECT(main_window), "tin_context");
    if (tin_context!=NULL){ /* dispose of any old tin structures. */
      set_output_choices_sensitive(main_window,FALSE);
      tin_destroy(tin_context);
      free(tin_context); tin_context=NULL;
      gtk_object_set_data(GTK_OBJECT(main_window), "tin_context", NULL);
    }
    /* Create the TIN. */
    tin_context=malloc(sizeof(tin_contextT));
    memset(tin_context,0,sizeof(tin_contextT));
    tin_create(tin_context,points,outrad);
    gtk_object_set_data (GTK_OBJECT(main_window), "tin_context", tin_context);
    /* tin plotting here. */
    draw_data(main_window);
    draw_triangles(main_window);
    draw_outline(main_window);
    set_output_choices_sensitive(main_window,TRUE);
  }else{
    /* else we have no data and should print an error message. */
  }
}/* on_create_tin_button_clicked */

gboolean on_main_window_delete_event(GtkWidget *widget,GdkEvent *event, gpointer user_data)
/* *******************************************************************
*  Free all the data structures on shutdown.
******************************************************************* */
{
  data_free(GTK_OBJECT(widget));
  gtk_main_quit();
  return FALSE;
}/* on_main_window_delete_event */


gboolean
on_tin_drawing_area_expose_event       (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
/* *******************************************************************
*   Redraw the screen from the backing rgbpad. Reacts whenever 
* any part of the drawing area is revealed or first appears.
******************************************************************* */
{
  GtkWidget *main_window;
  rgbpadT *rgb_pad;

  main_window=lookup_widget(GTK_WIDGET(widget),"main_window");
  rgb_pad=gtk_object_get_data (GTK_OBJECT(main_window), "rgb_pad");
  if (rgb_pad!=NULL){
    rgb_updatezone(widget, rgb_pad, event->area.x, event->area.y,
                   event->area.x+event->area.width, event->area.y+event->area.height);
  }
  return FALSE;
}/* on_tin_drawing_area_expose_event */

gboolean
on_tin_drawing_area_configure_event    (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
/* *******************************************************************
*   Redraw the data window when resized.
******************************************************************* */
{
  GtkWidget *main_window,*tin_drawing_area;
  rgbpadT *rgb_pad,*oldpad=NULL,newpad={0,0,0,0,NULL};
  int oldwdt=0,oldhgt=0;

  main_window=lookup_widget(GTK_WIDGET(widget),"main_window");
  tin_drawing_area = gtk_object_get_data(GTK_OBJECT(main_window),"tin_drawing_area");
  rgb_pad=gtk_object_get_data (GTK_OBJECT(main_window), "rgb_pad");
  oldpad = rgb_pad;

  rgb_setpad(&newpad,tin_drawing_area->allocation.width, tin_drawing_area->allocation.height);
  /* blank the new rgbpad with all white. */
  rgb_fillpad(&newpad,255,255,255);
  
  if (oldpad->bytesize>0){ /* if old rgbpad exists, copy old image to new rgbpad. */
    oldwdt=oldpad->wdt; oldhgt=oldpad->hgt;
    rgb_clonezone(&newpad,oldpad,0,0, 0,0,
        tin_drawing_area->allocation.width,tin_drawing_area->allocation.height);
    rgb_freepad(oldpad);
  }
  *rgb_pad = newpad;
  draw_data(main_window);
  draw_triangles(main_window);
  draw_outline(main_window);
  return (FALSE);
}/* on_tin_drawing_area_configure_event */


void
on_tin_drawing_area_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
/* *******************************************************************
******************************************************************* */
{
  GtkWidget *main_window;

  main_window=lookup_widget(GTK_WIDGET(widget),"main_window");
  set_output_choices_sensitive(main_window,FALSE);
  set_creation_choices_sensitive(main_window,FALSE);

  gtk_signal_connect_after (GTK_OBJECT (widget), "configure_event",
                            GTK_SIGNAL_FUNC (on_tin_drawing_area_configure_event),
                            NULL);
}/* on_tin_drawing_area_realize */


void
on_test_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
/* *******************************************************************
*  For debugging, not used.
******************************************************************* */
{
  GtkWidget *main_window,*tin_drawing_area;
  rgbpadT *rgb_pad=NULL,*oldpad=NULL,newpad={0,0,0,0,NULL};
  int oldwdt=0,oldhgt=0;

  printf("in on_test_button_clicked.\n");
  main_window=lookup_widget(GTK_WIDGET(button),"main_window");
  tin_drawing_area = gtk_object_get_data(GTK_OBJECT(main_window),"tin_drawing_area");
  rgb_pad=gtk_object_get_data (GTK_OBJECT(main_window), "rgb_pad");
  oldpad = rgb_pad;

  rgb_setpad(&newpad,tin_drawing_area->allocation.width, tin_drawing_area->allocation.height);
  /* blank the new rgbpad with all white. */
  rgb_fillpad(&newpad,255,255,255);
  
  if (oldpad->bytesize>0){ /* if old rgbpad exists, copy old image to new rgbpad. */
    oldwdt=oldpad->wdt; oldhgt=oldpad->hgt;
    rgb_clonezone(&newpad,oldpad,0,0, 0,0,
        tin_drawing_area->allocation.width,tin_drawing_area->allocation.height);
    rgb_freepad(oldpad);
  }
  *rgb_pad = newpad;
  draw_data(main_window);
  printf("leaving on_test_button_clicked.\n");
}/* on_test_button_clicked */


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
/* *******************************************************************
******************************************************************* */
{
  GtkWidget *main_window;
  main_window = lookup_widget(GTK_WIDGET(menuitem),"main_window");
  data_free(GTK_OBJECT(main_window));
  gtk_main_quit();
}/* on_exit_activate */

