/****************************************************************************
****************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>

#ifndef bool
  #define bool unsigned char
  #define true 0x01
  #define false 0x00
#endif
#ifndef byte
  #define byte unsigned char
#endif
#ifndef intdex
  #define intdex int
#endif
#ifndef maxlong /* biggest possible long integer. */
  #define maxlong 0x7FFFFFFF
#endif
/* identity flag to prevent declaration conflicts. */
#define _rgbtools 1

#define pixelsize 3

enum ClipState {Outside,Clipped,Inside};

typedef byte rgbT[pixelsize];

typedef struct rgbpadT{
  int wdt,hgt,bytewdt,bytesize;
  guchar *ras;
}rgbpadT;

/* graphical settings for painting with a single color. */
typedef struct gsettingsT{
  int raswidth;/* raster line width in pixels (not bytes). */
  /* a single raster line filled with the current color. */
  guchar *rascolor; // [10000*pixelsize];
  rgbT example;/* unchanging example of color. */
}gsettingsT;

/* Just a variable array of colors (gsettingsT).*/
typedef struct paletteT{
  int numcolors;
  gsettingsT *colors;
}paletteT;

void rgb_setpad(rgbpadT *pad,int wdt,int hgt);
void rgb_freepad(rgbpadT *pad);
void rgb_setcolor(gsettingsT *gs,guchar red,guchar green,guchar blue,int linewidth);
void rgb_freecolor(gsettingsT *gs);
void rgb_create_palette(paletteT *palette,int numcolors);
void rgb_free_palette(paletteT *palette);
void rgb_pack_palette(paletteT *palette);
void rgb_palette_set_spectral(paletteT *palette,int numcolors);

void rgb_putpixel(rgbpadT *rgbpad,int x,int y,guchar *rgb);
void rgb_gelbox(rgbpadT *pad,int x1,int y1,int x2, int y2,int r,int g,int b);
void rgb_gelcircle(rgbpadT *pad,int xcen,int ycen,int radius, int r,int g,int b);
void rgb_gelcircle_more(rgbpadT *pad,int xcen,int ycen,int radius, int r,int g,int b);
void rgb_drawline(rgbpadT *pad,gsettingsT *gs,int x1,int y1,int x2,int y2);
void rgb_fillpad(rgbpadT *pad,guchar r,guchar g,guchar b);
void rgb_fillzone(rgbpadT *pad,int x1,int y1,int x2, int y2,int r,int g,int b);
void rgb_clonezone(rgbpadT *pad1,rgbpadT *pad2,int xdest,int ydest, int x1,int y1,int x2,int y2);
void rgb_updatezone(GtkWidget *widget,rgbpadT *pad,int x1,int y1,int x2,int y2);
void rgb_slide_pad(rgbpadT *rgbpad,int dx,int dy);

void clipline(
  int *lx1,int *ly1,int *lx2,int *ly2,
  int rx1,int ry1,int rx2,int ry2,
  int *ClipState);

#ifdef __cplusplus
}
#endif
