/* **************************************************************************
  TINDEMO.C, a small program to demonstrate the TIN.C library.
  By John Atkeson, Jun 2000.  Tindemo.c is free for the taking.

  To compile:
  gcc -o tindemo tindemo.c tinlib.c space.c flexray.c -lm
  (or run the included 'ct.bat' script).

  Run in the same directory as the included xyz.txt file.
  
  WARNING: Before creating your own xyz text file, look at the structure of 
  the included one.  It has the total number of points at the very top.  
  The tinner needs this at the top to preset array sizes.
************************************************************************** */

#include "tinlib.h"

void print_triangles(tin_contextT *tcontext)
/* *******************************************************************
*  Print out all the triangles created by tinning.
* ******************************************************************* */
{
  int tcnt,vcnt;
  full_triT triangle; /* contains: double vtex[3][3], 3 xyz points */

  printf("tcontext->num_triangles:%li.\n",tcontext->num_triangles);

  for (tcnt=0;tcnt<tcontext->num_triangles;tcnt++){
    if(tin_get_triangle_full(tcontext,&triangle,tcnt)){

      printf("triangle:%li\n",tcnt);
      for (vcnt=0;vcnt<3;vcnt++){
        printf(
          "(x:%1.1lf y:%1.1lf z:%1.1lf) ",
          triangle.vtex[vcnt][0],
          triangle.vtex[vcnt][1],
          triangle.vtex[vcnt][2]);
      }
      printf("\n");
    }else{/* if it returned false, then we went out of range. */
      printf("Error, no triangle at index:%li.\n",tcnt);
    }

  }/* end for loop. */
}/* print_triangles */

int main()
/* *******************************************************************
*  This shows the essentials of using the TIN library.
* ******************************************************************* */
{
  PListT points;/* will contain all data xyz points. */
  tin_contextT tcontext;
  /* very big search radius, to produce a fully convex outline. */
  double bignumber=100000000000000000.0;

  /* read in all the survey xyz points. */
  tin_read_xyzs(&points,"xyz.txt");
  printf("%li survey xyz points.\n",points.NumPoints);

  /* Create the TIN. */
  printf("About to tin_create.\n");
  tin_create(&tcontext,&points,bignumber);

  /* show off all the triangles created. */
  print_triangles(&tcontext);

  /* save the triangles and outline to text files. */
  printf("Saving triangles to triangles.txt...\n");
  tin_save_triangles(&tcontext,"triangles.txt");

  printf("Saving outline to outline.txt...\n");
  tin_save_outline(&tcontext,"outline.txt");

  /* dispose of all of the tinner's interal structures. */
  printf("About to tin_destroy.\n");
  tin_destroy(&tcontext);

  /* Dispose of your own data. */
  FreePList(&points);

  printf("Done.\n");
  /* Bye! */
  return(0);
}/* main */


